import siptracklib.errors

from siptrackweb.views import helpers
from siptrackweb.forms import *

@helpers.authcheck
def add(request, parent_oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(parent_oid))
    pm.path(device)
    view = device.getParent('view')

    password_keys = view.listChildren(include = ['password key'])
    url = '/device/password/add/post/%s/' % (device.oid)
    pm.addForm(PasswordAddForm(password_keys), url,
            'add password', message = 'Adding password.',
            extraelements = 'autocomplete="off"')
    return pm.render()

@helpers.authcheck
def add_post(request, parent_oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(parent_oid))
    pm.path(device)
    view = device.getParent('view')

    password_keys = view.listChildren(include = ['password key'])
    url = '/device/password/add/post/%s/' % (device.oid)
    pm.addForm(PasswordAddForm(password_keys, request.POST), url,
            'add password', message = 'Adding password.',
            extraelements = 'autocomplete="off"')

    if not pm.form.is_valid():
        return pm.error()
    if not pm.form.cleaned_data['pw_password'] == pm.form.cleaned_data['validate']:
        return pm.error('passwords don\'t match')

    password_key = None
    if pm.form.cleaned_data['passwordkey'] != '__no-password-key__':
        password_key = pm.object_store.getOID(pm.form.cleaned_data['passwordkey'])

    password_string = pm.form.cleaned_data['pw_password']
    if len(password_string) == 0:
        password_string = helpers.generate_password()

    try:
        password = device.add('password', password_string, password_key)
    except siptracklib.errors.SiptrackError, e:
        return pm.error(str(e))

    password.attributes['username'] = pm.form.cleaned_data['pw_username']
    if len(pm.form.cleaned_data['description']) > 0:
        password.attributes['description'] = pm.form.cleaned_data['description']

    return pm.redirect('device.display', (device.oid,))

@helpers.authcheck
def delete(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    password = pm.object_store.getOID(oid)
    pm.addForm(DeleteForm(), '/device/password/delete/post/%s/' % (password.oid),
            'remove password', message = 'Removing password.')
    pm.path(password)

    return pm.render()

@helpers.authcheck
def delete_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    password = pm.object_store.getOID(oid)
    device = password.parent
    pm.path(password)
    pm.addForm(DeleteForm(request.POST), '/device/password/delete/post/%s/' % (password.oid),
            'remove password', message = 'Removing password.')

    if not pm.form.is_valid():
        return pm.render()
    password.delete()

    return pm.redirect('device.display', (device.oid,))

