from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import Context, loader
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from siptracklib.utils import object_by_attribute
import siptracklib.errors
import helpers
from siptrackweb.forms import *
from siptrackweb.views import attribute

@helpers.authcheck
def index(request, parent_oid):
    pm = helpers.PageManager(request, 'stweb/views/passwordkeys/index.html')
    parent = pm.setVar('parent', pm.object_store.getOID(parent_oid))
    pm.path(parent)
    pm.section('password')

    pm.render_var['password_key_list'] = parent.listChildren(include = ['password key'])
    return pm.render()

@helpers.authcheck
def display(request, oid):
    pm = helpers.PageManager(request, 'stweb/views/passwordkeys/display.html')

    pk = pm.setVar('password_key', pm.object_store.getOID(oid))
    pm.path(pk)
    pm.render_var['parent'] = pk.parent
    pm.render_var['password_key_list'] = pk.parent.listChildren(include = ['password key'])
    pm.render_var['attribute_list'] = attribute.parse_attributes(pk)
    return pm.render()

@helpers.authcheck
def add(request, parent_oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    parent = pm.setVar('parent', pm.object_store.getOID(parent_oid))
    pm.path(parent)

    pm.render_var['password_key_list'] = parent.listChildren(include = ['password key'])
    url = '/passwordkey/add/post/%s/' % (parent.oid)
    pm.addForm(PasswordKeyAddForm(), url, 'add password key')
    return pm.render()

@helpers.authcheck
def add_post(request, parent_oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    parent = pm.setVar('parent', pm.object_store.getOID(parent_oid))
    pm.path(parent)

    pm.render_var['password_key_list'] = parent.listChildren(include = ['password key'])
    url = '/passwordkey/add/post/%s/' % (parent.oid)
    pm.addForm(PasswordKeyAddForm(request.POST), url, 'add password key')
    if not pm.form.is_valid():
        return pm.error()
    if not pm.form.cleaned_data['key'] == pm.form.cleaned_data['validate']:
        return pm.error('keys don\'t match')

    pk = parent.add('password key', pm.form.cleaned_data['key'])
    pk.attributes['name'] = pm.form.cleaned_data['name']
    if len(pm.form.cleaned_data['description']) > 0:
        pk.attributes['description'] = pm.form.cleaned_data['description']

    return pm.redirect('passwordkey.display', (pk.oid,))

@helpers.authcheck
def delete(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    pk = pm.setVar('password_key', pm.object_store.getOID(oid))
    pm.path(pk)
    pm.render_var['parent'] = pk.parent
    pm.render_var['password_key_list'] = pk.parent.listChildren(include = ['password key'])
    url = '/passwordkey/delete/post/%s/' % (oid)
    pm.addForm(DeleteForm(), url,
            'remove password key', message = 'Removing password key.')
    return pm.render()

@helpers.authcheck
def delete_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    pk = pm.object_store.getOID(oid)
    pm.path(pk)
    parent_oid = pk.parent.oid
    url = '/passwordkey/delete/post/%s/' % (oid)
    pm.addForm(DeleteForm(request.POST), url,
            'remove password key', message = 'Removing password key.')
    if not pm.form.is_valid():
        return pm.render()
    try:
        pk.delete()
    except siptracklib.errors.SiptrackError, e:
        return pm.error(str(e))

    return pm.redirect('passwordkey.index', (parent_oid,))

